import typer
from pathlib import Path
import os
import requests
import json
import yaml
from ..config import URL_CREATE_EXPERIMENT
from ..utils import load_yaml_doc
from ..utils import get_token
from ..utils import get_exp_token
import inquirer

from .helper import (
    final_auto_al_ml_driven_ruled_based_call,
    get_auto_al_info,
    get_ml_driven_info,
    create_experiment_api_request,
)
from datetime import datetime


def get_experiment_type_and_dict_manuevers(yaml_file: Path):
    """Creates 4 types of experiment.
       1. Auto curation
       2. ML Driven
       3. Rule Based
       4. Manual Curation


    Args:
        yaml_dict (dict): _description_

    Raises:
        typer.Exit: _description_

    Returns:
        _type_: _description_
    """

    token = get_token()
    if not token:
        raise typer.Exit()
    yaml_dict = load_yaml_doc(yaml_file)

    questions = [
        inquirer.List(
            "type",
            message="What is the type of curation",
            choices=["auto_al", "proprietary", "rule_based", "manual_curation"],
        )
    ]
    answers = inquirer.prompt(questions)
    type = answers.get("type")
    if type == "auto_al":
        is_alectio_qs = False
        yaml_dict["type"] = type
        yaml_dict["limits"] = False
        yaml_dict["is_alectio_qs"] = is_alectio_qs
        yaml_dict["n_loops"] = 10
        yaml_dict["epochs"] = 0
        yaml_dict["batch_size"] = 0
        final_dict = get_auto_al_info(yaml_dict)
        return_value = final_auto_al_ml_driven_ruled_based_call(final_dict, token)

    elif type == "proprietary":
        is_alectio_qs = True
        yaml_dict["type"] = type
        yaml_dict["limits"] = False
        yaml_dict["is_alectio_qs"] = is_alectio_qs
        yaml_dict["epochs"] = 0
        yaml_dict["batch_size"] = 0

        final_dict = get_ml_driven_info(yaml_dict)
        return_value = final_auto_al_ml_driven_ruled_based_call(final_dict, token)

    elif type == "rule_based":
        type = "design_own"
        is_alectio_qs = True
        yaml_dict["type"] = type
        yaml_dict["limits"] = False
        yaml_dict["is_alectio_qs"] = is_alectio_qs
        yaml_dict["epochs"] = 0
        yaml_dict["batch_size"] = 0

        final_dict = get_ml_driven_info(yaml_dict)
        return_value = final_auto_al_ml_driven_ruled_based_call(final_dict, token)

    elif type == "manual_curation":
        n_loops = typer.prompt(
            f"How many loops you would like to run for the experiment ?/n"
        )
        yaml_dict["n_loops"] = n_loops
        yaml_dict["type"] = type
        for qs in yaml_dict["qs"]:
            check_qs = yaml_dict["qs"][qs]["enable"]
            if check_qs:
                temp_data_loaded = yaml_dict.copy()
                temp_data_loaded["qs"] = [qs]
                temp_data_loaded["name"] = f'{temp_data_loaded["name"]}_{qs}'
                date = datetime.now()
                date = int(date.strftime("%Y%m%d%H%M%S"))
                temp_data_loaded["date"] = date
                result = create_experiment_api_request(token, temp_data_loaded)
                status = result["message"]
                experiment_id = result["data"]
                message = typer.style(
                    f"QS: {qs}\nstatus: {status}\nExperiment ID: {experiment_id}",
                    fg=typer.colors.GREEN,
                )
                typer.echo(message)

    return True


def run_experiment(python_file: Path, experiment_file: Path):
    """Run Experiment

    Args:
        python_file (python_file): _description_
        experiment_file (experiment_file): _description_

    Raises:
        typer.Exit: _description_
    """
    token = get_token()
    if not token:
        raise typer.Exit()
    data_loaded = load_yaml_doc(experiment_file)
    if data_loaded["type"] == "manual_curation":
        pass
    elif data_loaded["type"] == "auto_al":
        exp_token = get_exp_token(token, data_loaded["experiment_id"])["token"]
        os.system(f"python {python_file} --token {exp_token}")
        pass



from random import choices
import requests
from pathlib import Path
import typer
from rich.console import Console
from datetime import datetime
import json
import inquirer
from alectiocli.src import console
from ..utils import get_token, load_yaml_doc
from .marketplace_recommendation import select_market_place_recommendation
from ..config import BACKEND_URL, AUTOLABEL_CLASS, INSTRUCTION_UPLOAD_URL
from ..utils import get_token
from .helper import upload_instruction_doc

rich_console = Console()


def ask_qualitycheck_questions():

    request_body = {}
    request_body["anomaly_criteria"] = {}
    console.message("Select quality checking for the labeling result: ")
    choices = [
        "Automatic quality checking",
        "Proceed without quality checking",
        "Pause without quality checking",
        "Relabel certain amount of data without quality checking and then proceed",
    ]
    quality_check_type_options = [
        inquirer.List(
            "quality_check_type",
            message="Select quality check method:",
            choices=choices,
        ),
    ]
    quality_check_type = inquirer.prompt(quality_check_type_options)[
        "quality_check_type"
    ]
    if quality_check_type == choices[3]:
        request_body["type"] = "relabel_amount_then_proceed"
        value = int(typer.prompt(f"What percentage of data to be relabeled (eg. 30)? "))
        request_body["amount_percentage"] = value
    if quality_check_type == choices[1]:
        request_body["type"] = "proceed_without_checking"
        request_body["amount_percentage"] = None
    if quality_check_type == choices[2]:
        request_body["type"] = "pause"
        request_body["amount_percentage"] = None

    if quality_check_type == choices[0]:
        request_body["type"] = "auto_quality_checking"
        request_body["amount_percentage"] = None
        request_body["action"] = {}

        console.message("Define anomaly criteria")
        console.info(
            "Alectio will check each labels and find anomalies based on the criteria you defined."
        )
        criteria_choice = ["Width", "Height", "Area", "Aspect Ratio"]
        criteria_choice_types = inquirer.prompt(
            [
                inquirer.Checkbox(
                    "criteria_types",
                    message="Select criteria types: ",
                    choices=criteria_choice,
                )
            ]
        )

        def ask_criteria_range(criteria_name, retry_count=0):
            retry_count = retry_count + 1

            if retry_count > 4:
                console.error("you exceed retry limit, try sometime letter !")

            lower_point = int(
                typer.prompt(f"Lowest value of {criteria_name}: (eg. 40): ")
            )
            if lower_point < 0:
                console.warning(
                    "Put the value again\nLower range value should be more than or equals to 0"
                )
                ask_criteria_range(criteria_name, retry_count)
            max_point = int(typer.prompt(f"Max value of {criteria_name}: (eg. 40): "))
            if max_point > 100 and max_point < lower_point:
                console.warning(
                    "Put the value again\nMax value should be less than 100 and greater than above Lower range value"
                )
                ask_criteria_range(criteria_name, retry_count)

            return [lower_point, max_point]

        if "Width" in criteria_choice_types["criteria_types"]:
            value_range = ask_criteria_range("Width")
            request_body["anomaly_criteria"]["width"] = value_range

        if "Height" in criteria_choice_types["criteria_types"]:
            value_range = ask_criteria_range("Height")
            request_body["anomaly_criteria"]["height"] = value_range
        if "Area" in criteria_choice_types["criteria_types"]:
            value_range = ask_criteria_range("Area")
            request_body["anomaly_criteria"]["area"] = value_range
        if "Aspect Ratio" in criteria_choice_types["criteria_types"]:
            value_range = ask_criteria_range("Aspect Ratio")
            request_body["anomaly_criteria"]["aspect_ratio"] = value_range

        # Define when to take action
        console.message("Define when to take action: ")
        whenaction_choice = [
            "When any anomaly is detected",
            "when X% labels are detected as anomalies",
        ]
        chosen_whenaction = inquirer.prompt(
            [
                inquirer.List(
                    "chosen_whenaction",
                    message="When to take any action: ",
                    choices=whenaction_choice,
                )
            ]
        )["chosen_whenaction"]
        if chosen_whenaction == whenaction_choice[1]:
            value = int(typer.prompt(f"Put the value of X: (eg. 40): "))
            request_body["action"]["when"] = value
        else:
            request_body["action"]["when"] = 0

        # Define what action to take
        console.message("Define what action to take: ")
        action_choices = ["Pause and warn me", "Relabel automatically"]
        chosen_action = inquirer.prompt(
            [
                inquirer.List(
                    "chosen_action",
                    message="What action do you want to take: ",
                    choices=action_choices,
                )
            ]
        )["chosen_action"]

        if chosen_action == action_choices[1]:
            value = int(
                typer.prompt(f"Define maximum number of relabeling processes (eg. 3): ")
            )
            request_body["action"]["what"] = "relable_automatic"
            request_body["action"]["number_of_relabel_times"] = value
        else:
            request_body["action"]["what"] = "pause"
            request_body["action"]["number_of_relabel_times"] = None
    return request_body


def create_hybrid_labeling_task(yaml_file: Path):
    # load yaml file
    config = load_yaml_doc(yaml_file)
    # make request body
    request_body = {}
    request_body["task_name"] = config["task_name"]
    project_id = config["project_id"]
    request_body["task_description"] = config["task_description"]
    task_type = config["task_type"]
    request_body["task_type"] = task_type
    annotation_type = config["annotation_type"]
    request_body["annotation_type"] = annotation_type
    classes = config["classes"]

    choices = [
        "All my data",
        "Randomly select X% of my data",
        "Specify range of indexes",
        "From an experiment",
    ]
    selection_type_options = [
        inquirer.List(
            "selection_type",
            message="Please specify the records you’d like to label",
            choices=choices,
        ),
    ]
    selection_type = inquirer.prompt(selection_type_options)["selection_type"]
    if selection_type == choices[0]:
        chosen_selection_type = "all"
        selection_value = 0
    if selection_type == choices[1]:
        chosen_selection_type = "random"
        selection_value = int(
            typer.prompt(
                "What parcentage(0-100) of data, you want to randomly select? eg. 40"
            )
        )
    if selection_type == choices[2]:
        chosen_selection_type = "range"
        index_low = int(typer.prompt("Starting index(eg. 100): "))
        index_high = int(typer.prompt("Ending index(eg. 600): "))
        selection_value = [index_low, index_high]

    if selection_type == choices[3]:
        chosen_selection_type = "experiment"
        console.error("This is not suppported yet")
        typer.Exit()
        selection_value = 0

    # labelling methodology
    labeling_methodology = config["labeling_methodology"]["type"]
    if labeling_methodology == "market_place":
        multi_judgement_count = config["labeling_methodology"]["multi_judgement_count"]
        instruction_file = config["labeling_methodology"]["instruction_file"]
        upload_instruction_doc(project_id, Path(instruction_file))
        request_body["instruction_file"] = "s3_key"

        # prompt user with options
        console.message("How do you want to select Labeling Partner?")
        console.message("1. Alectio Recommendation")
        console.message("2. Own Choice")
        recommendation_choice = typer.prompt("select (1/2): ", default="1")
        marketplace_chosen = select_market_place_recommendation(
            annotation_type=annotation_type,
            project_id=project_id,
            task_type=task_type,
            choice=recommendation_choice,
        )
    else:
        multi_judgement_count = 1
        instruction_file = ""
        marketplace_chosen = None
        request_body["instruction_file"] = ""

    request_body["methods"] = []
    request_body["methods"].append(
        {
            "labeling_method": labeling_methodology,
            "classes": classes,
            "selection_type": chosen_selection_type,
            "value": selection_value,
            "number_of_annotators_per_record": multi_judgement_count,
            "partners": marketplace_chosen,
        }
    )
    # auto labeling
    token = get_token()

    def find_autolabel_support(task_type, class_name):
        headers = {"Authorization": str(token), "content-type": "application/json"}
        api_url = f"{AUTOLABEL_CLASS}/{task_type}/classes/{class_name}"

        post_req_result = requests.post(api_url, headers=headers)
        if post_req_result.status_code == 200:
            response = post_req_result.json()
            if "class_name" in response["data"]:
                return response["data"]["class_name"]
        return False

    with rich_console.status(
        "[bold green]Finding AutoAL support of classes..."
    ) as status:
        auto_labeling_supported_class = []
        for c in classes:
            if find_autolabel_support(task_type=task_type, class_name=c):
                auto_labeling_supported_class.append(c)

    request_body["methods"].append(
        {
            "labeling_method": "auto_labeling",
            "classes": auto_labeling_supported_class,
            "selection_type": chosen_selection_type,
            "value": selection_value,
        }
    )
    request_body["instruction_file"] = instruction_file

    # quality check parameters
    quality_checking = ask_qualitycheck_questions()
    request_body["quality_checking"] = quality_checking

    with rich_console.status(
        "[bold green]Alectio is creating a Hybrid labeling task for you ..."
    ) as status:
        done = False
        headers = {
            "Authorization": str(token),
            "content-type": "application/json",
        }
        payload = request_body
        while not done:
            BACKEND_URL = "https://devbackend.alectio.com"
            post_req_result = requests.post(
                f"{BACKEND_URL}/project/{project_id}/task/create",
                headers=headers,
                data=json.dumps(payload),
            )
            if post_req_result.status_code == 200:
                done = True
                console.success(
                    "Hybrid labeling task is created, check Alectio Portal !"
                )



import typer
from pathlib import Path
from rich.console import Console
from ..utils import get_token
from datetime import datetime
from ..config import INSTRUCTION_UPLOAD_URL
from alectiocli.src import console
import requests

rich_console = Console()


def upload_instruction_doc(project_id, doc_path: Path):
    if doc_path.is_file() and doc_path.suffix in [".pdf", ".txt", ".text"]:
        with rich_console.status(
            "[bold green]Uploading labeling instruction file..."
        ) as status:
            token = get_token()

            headers = {"Authorization": str(token)}
            files = {"instructions": open(doc_path, "rb")}
            payload = {
                "project_id": project_id,
                "date": datetime.now().strftime("%d/%m/%Y"),
            }
            post_req_result = requests.post(
                INSTRUCTION_UPLOAD_URL,
                headers=headers,
                files=files,
                data=payload,
            )
            if post_req_result.status_code == 200:
                console.success("Instruction uploaded successfully")
            else:
                console.error("Instruction upload failed")
                raise typer.Exit()
    else:
        console.error(
            "Wrong instruction file type: Please Enter a [dot]pdf/[dot]txt/[dot]text file"
        )
        raise typer.Exit()



APP_NAME = "AlectioCLI"
SSO_URL = "http://alectio.localhost.com:4200/SSO/login/"
BACKEND_URL = "https://devbackend.alectio.com"
URL_CREATE_PROJECT = f"{BACKEND_URL}/projects/create"
URL_CREATE_EXPERIMENT = f"{BACKEND_URL}/projects/addExperimentsInfo"
URL_TOKEN_VALIDATE = f"{BACKEND_URL}/users/projects"
URL_GET_EXP_TOKEN = f"{BACKEND_URL}/experiments/get_token"
INSTRUCTION_UPLOAD_URL = f"{BACKEND_URL}/instructions/upload"
CLASSES_URL = f"{BACKEND_URL}/project/classes"
MARKETPLACE_RECOMMENDATION_URL = f"{BACKEND_URL}/partners/recommended"
AUTOLABEL_CLASS = f"{BACKEND_URL}/autoLabel/models/type"
INSTRUCTION_UPLOAD_URL = f"{BACKEND_URL}/instructions/upload"
PROJECT_LIST_URL = f"{BACKEND_URL}/users/projects"
DELETE_PROJECT_URL = f"{BACKEND_URL}/projects/delete"



import os
import json
import typer
import yaml
import requests
from pathlib import Path
from .config import APP_NAME, URL_TOKEN_VALIDATE, URL_GET_EXP_TOKEN


def get_config_path():
    app_dir = typer.get_app_dir(APP_NAME)

    config_path: Path = Path(app_dir)
    if not config_path.exists():
        config_path.mkdir(parents=True, exist_ok=True)
    return config_path


def get_token():
    config_path = get_config_path()
    token_path = config_path / ".token"
    if not token_path.exists():
        typer.echo(
            "Token doesn't exist! Please use `alectio-cli login` to get the token"
        )
        return False
    else:
        token = token_path.read_text()

        status = validate_token(token)

        if status["status"] == "success":
            return token
        else:
            token_path.unlink()
            typer.echo("Invalid token. Please use `alectio-cli login` to get the token")
            return False


def validate_token(token):
    headers = {"Authorization": str(token)}
    try:
        post_req_result = requests.get(URL_TOKEN_VALIDATE, headers=headers)

    except:
        return {"status": "failed"}
    if post_req_result.status_code == 200:
        return post_req_result.json()
    else:
        return {"status": "failed"}


def load_yaml_doc(yaml_file: Path):
    """
    Loads yaml file into a python dict and checks for validation
    Note : ` yaml.safe_load(stream)` is used to remove trailing zeroes from yaml file.
    """
    if yaml_file.is_file:
        if os.path.splitext(yaml_file)[1] == ".yaml":
            with open(yaml_file, "r") as stream:
                try:
                    return yaml.safe_load(stream)
                except yaml.YAMLError as exception:
                    raise exception
        else:
            message = typer.style(
                "Wrong file type: Please Enter a [dot]yaml file",
                fg=typer.colors.WHITE,
                bg=typer.colors.RED,
            )
            raise typer.Exit(message)
    else:
        message = typer.style("Not a file", fg=typer.colors.WHITE, bg=typer.colors.RED)
        raise typer.Exit(message)


def get_exp_token(token, exp_id):
    headers = {"Authorization": str(token), "Content-type": "application/json"}
    data = {"experiment_id": exp_id}
    try:
        post_req_result = requests.post(
            URL_GET_EXP_TOKEN, headers=headers, data=json.dumps(data)
        )
    except:
        return {"status": "failed"}
    if post_req_result.status_code == 200:
        return post_req_result.json()
    else:
        print(post_req_result.text)
        return {"status": "failed"}



import typer
from pathlib import Path
import os
import requests
from ..config import URL_CREATE_PROJECT
from ..utils import get_token, load_yaml_doc
from .helper import (
    create_sample_yaml_for_experiment,
    manual_curation_experiment,
    create_hybrid_labeling_yaml_file,
)


def create_project_api_request(token, data, files):

    headers = {"Authorization": str(token)}
    post_req_result = requests.post(
        URL_CREATE_PROJECT, headers=headers, files=files, data=data
    )
    return post_req_result.json()


def create_project_on_prem(yaml_file: Path, label_map: Path):
    """
    Creates project for On prem

    :param yaml_file: Takes input a sample yaml file
    :param yaml_file: Takes input a json file containing classes/ label_map

    """
    token = get_token()

    if not token:
        raise typer.Exit()
    data_loaded = load_yaml_doc(yaml_file)
    if label_map.is_file:
        if os.path.splitext(label_map)[1] == ".json":
            files = {"class_labels": open(label_map, "rb")}
        else:
            message = typer.style(
                "Wrong file type: Please Enter a [dot]json file",
                fg=typer.colors.WHITE,
                bg=typer.colors.RED,
            )
            return typer.echo(message)
            raise typer.Exit()
    else:
        message = typer.style("Not a file", fg=typer.colors.WHITE, bg=typer.colors.RED)
        return typer.echo(message)

    result = create_project_api_request(token, data_loaded, files)

    status = result["message"]
    project_id = result["project_id"]
    if status == "project successfully created":
        folder_name = "Alectio_cli_sample_yamls"
        current_working_directory = os.getcwd()
        folder_path = os.path.join(current_working_directory, folder_name)
        if not os.path.exists(folder_path):
            os.mkdir(folder_path)
        create_sample_yaml_for_experiment(folder_path, project_id)
        manual_curation_experiment(folder_path, project_id)
        create_hybrid_labeling_yaml_file(folder_path, project_id)

    messge = typer.style(
        f"status: {status}\nProject ID: {project_id}", fg=typer.colors.GREEN
    )

    typer.echo(messge)
    typer.secho(f"Sample yaml files are located at {folder_path}", fg=typer.colors.BLUE)
    return True



from random import choices
import typer
from ..utils import get_exp_token

from pathlib import Path
import yaml
import os
from ..utils import load_yaml_doc
import requests
from ..config import URL_CREATE_EXPERIMENT
import json
import inquirer
from datetime import datetime


def get_auto_al_info(yaml_dict):
    """_summary_

    Args:
        yaml_dict (dict): Takes a dictinoary as dict

    Returns:
        _type_: _description_
    """

    max_records = yaml_dict["n_records"]
    limit_value = typer.prompt(
        f"Define a custom stopping criteria for your experiment?\nEnter integers value from 1 to {max_records} records present"
    )
    typer.echo("\n")

    if int(limit_value) > int(max_records):
        typer.echo(
            f"Limit value is greater than maximum records i.e. {max_records}\nPlease re-enter the limit value"
        )
        get_auto_al_info(yaml_dict)
    else:
        confirm = typer.confirm("Confirm ")
        info_dict = {}
        if not confirm:

            typer.echo("Taking to the previous step")
            get_auto_al_info(yaml_dict)

        info_dict = {
            "auto_al_info": {
                "type": "limit",
                "threshold": {
                    "limit_type": "records",
                    "limit_value": limit_value,
                    "limit_unit": None,
                },
            }
        }
        yaml_dict["auto_al_info"] = info_dict["auto_al_info"]

    return yaml_dict


def get_ml_driven_info(yaml_dict):
    max_records = yaml_dict["n_records"]
    n_loops = typer.prompt(
        f"How many loops you would like to run for the experiment ?/n"
    )
    limit = typer.prompt(f"Define limit per loop")
    total_selected_records = int(n_loops) * int(limit)
    if int(total_selected_records) > int(max_records):
        typer.echo(
            " you will run out of record before the end, Choose a lesser value of limit per loop"
        )
        get_ml_driven_info(yaml_dict)

    else:
        confirm = typer.confirm("Confirm ")
        info_dict = {}
        if not confirm:

            typer.echo("Taking to the previous step")
            get_ml_driven_info(yaml_dict)

        yaml_dict["n_records"] = total_selected_records
        yaml_dict["n_loops"] = n_loops

    return yaml_dict


def create_experiment_api_request(token, data):
    headers = {"Authorization": str(token), "Content-type": "application/json"}
    post_req_result = requests.post(
        URL_CREATE_EXPERIMENT, headers=headers, data=json.dumps(data)
    )
    return post_req_result.json()


def final_auto_al_ml_driven_ruled_based_call(final_dict, token):

    date = datetime.now()
    date = int(date.strftime("%Y%m%d%H%M%S"))
    final_dict["date"] = date
    result = create_experiment_api_request(token, final_dict)
    status = result["message"]
    project_id = result["data"]
    message = typer.style(
        f"status: {status}\Experiment ID: {project_id}", fg=typer.colors.GREEN
    )
    typer.echo(message)
    return True



import typer


def error(msg: str):
    msg = "❌  " + msg + "\n"
    message = typer.style(msg, fg=typer.colors.RED)
    typer.echo(message)


def success(msg: str):
    msg = "✅  " + msg + "\n"
    message = typer.style(msg, fg=typer.colors.GREEN)
    typer.echo(message)


def info(msg: str):
    msg = "ℹ️  " + msg + "\n"
    message = typer.style(msg, fg=typer.colors.BLUE)
    typer.echo(message)


def warning(msg: str):
    msg = "⚠️  " + msg + "\n"
    message = typer.style(msg, fg=typer.colors.YELLOW)
    typer.echo(message)


def message(msg: str):
    message = typer.style(msg, fg=typer.colors.CYAN)
    typer.echo(message)



import requests
from ..utils import get_token
from ..config import PROJECT_LIST_URL
from alectiocli.src import console
from rich.console import Console
from rich.table import Table

rich_console = Console()


def show_table(response):
    table = Table(title="Your all projects")

    table.add_column("Proj Name", justify="right", style="cyan", no_wrap=True)
    table.add_column("Proj ID", style="cyan", no_wrap=False)
    table.add_column("Dataset Source", style="cyan", no_wrap=False)
    table.add_column("Task Type", style="cyan", no_wrap=False)
    table.add_column("Data Format", style="cyan", no_wrap=False)
    table.add_column("Train Length", style="cyan", no_wrap=False)

    for row in response["data"]:
        table.add_row(
            str(row["name"]),
            str(row["SK"].replace("PROJ#", "")),
            str(row["dataset_source"]),
            str(row["labeling_instructions"]["task_type"].replace("_", " ").upper()),
            str(row["data_info"]["format"]),
            str(row["train"]["length"]),
        )

    rich_console.print(table)


def get_project_list():
    with rich_console.status(
        "[bold green]Collecting the list of all your projects..."
    ) as status:
        token = get_token()
        headers = {"Authorization": str(token)}
        try:
            post_req_result = requests.get(PROJECT_LIST_URL, headers=headers)
        except:
            console.error("Something went wrong !!")
        if post_req_result.status_code == 200:
            show_table(post_req_result.json())
            # return post_req_result.json()

        else:
            console.error("Something went wrong !!")



import collections
import yaml
import os
import ruamel.yaml


def create_sample_yaml_for_experiment(folder_path, project_id):
    """Create sample yaml for experiment

    Args:
        project_id (string): project id
    values:
    name,n_records,seeds,labeling_type,is_sdk_setup,
    is_curr_fully_labeled,assoc_labeling_task_id
    """

    yaml_str = """# sample code 
# QS list is empty for auto_al,ml_driven curation
        project_id: 
        name: 'test_experiment'
        n_records: 100
        limits: False
        qs: [] 
        seed: 120
        labeling_type: pre_labeled
        is_sdk_setup: False
        is_curr_fully_labeled: False
        assoc_labeling_task_id:
"""

    yaml_file = ruamel.yaml.YAML()
    code = yaml_file.load(yaml_str)
    code["project_id"] = str(project_id)
    # current_working_directory=os.getcwd()

    # print(current_working_directory)
    with open(os.path.join(folder_path, "experiment.yaml"), "w+") as outfile:
        ruamel.yaml.dump(code, outfile, Dumper=ruamel.yaml.RoundTripDumper)
    return "experiment file created"


def manual_curation_experiment(folder_path, project_id):
    """Create sample yaml for experiment of manual curation

    Args:
        project_id (string): project id
    """

    yaml_str = """# sample code
    project_id: 
    name: 'New Experiment Manual curation test'
    n_records: 100
    limits: False

    qs:
        confidence:
            enable: True
            range:
                start: 0.5
                stop: 1
        entropy:
            enable: True
            range:
                start: 0.5
                stop: 1
        margin:
            enable: True
            range:
                start: 0.5
                stop: 1

    seed: 120
    labeling_type: pre_labeled
    is_sdk_setup: False
    is_curr_fully_labeled: False
    assoc_labeling_task_id:
"""

    yaml_file = ruamel.yaml.YAML()
    code = yaml_file.load(yaml_str)
    code["project_id"] = str(project_id)
    # current_working_directory=os.getcwd()

    # print(current_working_directory)
    with open(
        os.path.join(folder_path, "sample_experiment_manual_curation.yaml"), "w+"
    ) as outfile:
        ruamel.yaml.dump(code, outfile, Dumper=ruamel.yaml.RoundTripDumper)
    return " manual curation experiment file created"


def create_hybrid_labeling_yaml_file(folder_path, project_id):

    """Create sample yaml for hybrid labeling

    Args:
        project_id (string): project id
    """
    yaml_str = """# sample code
    task_name: "Hybrid Labeling for Object Detection"
    project_id: 
    task_description: "This is a YOLO V5 model for brand logo detection"
    task_type: "2d_object_detection"
    annotation_type: "bounding_boxes"
    classes: ["dog", "cat", "horse", "rat", "bird", "dog"]

    labeling_methodology:
        type: market_place
        multi_judgement_count: 5
        instruction_file: "converted_compressed.pdf"

    quality_check:
        type: auto_quality_checking
        anomaly_criteria:
            width: [14, 40]
            height: [14, 40]
            area: [14, 40]
            aspect_ratio: [14, 40]
        action:
            when: 20
            what: relabel_automatic
            number_of_relabel_times: 4
"""

    yaml_file = ruamel.yaml.YAML()
    code = yaml_file.load(yaml_str)
    code["project_id"] = str(project_id)
    with open(
        os.path.join(folder_path, "sample_hybrid_labeling.yaml"), "w+"
    ) as outfile:
        ruamel.yaml.dump(code, outfile, Dumper=ruamel.yaml.RoundTripDumper)
    return "hybrid labeling experiment file created"



import typer
from .src.login import retrieve_token
from .src.project.project import create_project_on_prem
from .src.project.project_list import get_project_list
from .src.project.delete_project import delete_project
from .src.experiment.experiment import (
    get_experiment_type_and_dict_manuevers,
    run_experiment,
)

from .src.hybrid_labeling.hybrid_labeling import create_hybrid_labeling_task
from pathlib import Path

app = typer.Typer()

project_app = typer.Typer()
app.add_typer(project_app, name="project")
experiment_app = typer.Typer()
app.add_typer(experiment_app, name="experiment")


@app.callback()
def callback():
    """
    AlectioCLI
    """


@app.command()
def login(refresh: bool = False):
    """
    Login to Alectio
    """
    status = retrieve_token(refresh)
    if not status:
        typer.echo("Error storing token")


@project_app.command("create")
def project_creation(yaml_file: Path, label_file: Path):

    """
    Creating Project
    """

    message = typer.style(
        "Creating Project ...",
        fg=typer.colors.BLUE,
    )
    typer.echo(message)
    status = create_project_on_prem(yaml_file, label_file)


@project_app.command("hybrid-labeling")
def create_hybrid_labeling(yaml_file: Path):
    """
    Creating Hybrid Labeling Task
    """

    message = typer.style(
        "Creating Hybrid Labeling Task ...",
        fg=typer.colors.MAGENTA,
    )
    typer.echo(message)
    status = create_hybrid_labeling_task(yaml_file)


@experiment_app.command("create")
def experiment_creation(yaml_file: Path):

    """
    Creating Experiment
    """

    message = typer.style(
        "Creating Experiment ...",
        fg=typer.colors.MAGENTA,
    )
    typer.echo(message)
    status = get_experiment_type_and_dict_manuevers(yaml_file)


@experiment_app.command("run")
def experiment_run(python_file: Path, experiment_file: Path):

    """
    Run the Experiment
    """

    message = typer.style(
        "Running the Experiment ...",
        fg=typer.colors.MAGENTA,
    )
    typer.echo(message)
    status = run_experiment(python_file, experiment_file)


@project_app.command("list")
def show_project_list():
    """
    show project list(all private project)
    """
    get_project_list()


@project_app.command("delete")
def delete_proj(proj_id: str):
    """
    Delete Project
    """
    delete_project(proj_id=proj_id)



from rich.console import Console
from rich.table import Table
import typer
import requests
from alectiocli.src import console
from ..config import MARKETPLACE_RECOMMENDATION_URL
from ..utils import get_token
import json
import inquirer


# rich console
rich_console = Console()
market_place_list = [
    [
        {
            "partner_keyname": "HITL",
            "calc_price_per_row": 0.0,
            "calc_accuracy_per_row": 95.0,
            "calc_time_per_row": 36.0,
            "n_annotators": 200,
        },
        1548.0,
    ],
    [
        {
            "partner_keyname": "INFOSCRIBE",
            "calc_price_per_row": 0.02,
            "calc_accuracy_per_row": 95.0,
            "calc_time_per_row": 36.0,
            "n_annotators": 1000,
        },
        1459.0,
    ],
    [
        {
            "partner_keyname": "DAIVERGENT",
            "calc_price_per_row": 0.2,
            "calc_accuracy_per_row": 98.0,
            "calc_time_per_row": 10.0,
            "n_annotators": 1,
        },
        773.0,
    ],
    [
        {
            "partner_keyname": "RBC",
            "calc_price_per_row": 0.05,
            "calc_accuracy_per_row": 90.0,
            "calc_time_per_row": 36.0,
            "n_annotators": 200,
        },
        670.0,
    ],
    [
        {
            "partner_keyname": "STEPWISE",
            "calc_price_per_row": None,
            "calc_accuracy_per_row": 95.0,
            "calc_time_per_row": 36.0,
            "n_annotators": 1,
        },
        550.0,
    ],
]


def show_table(market_place_data):
    table = Table(title="Avaailable MarketPlace Partners")

    table.add_column("Partner Name", justify="right", style="cyan", no_wrap=True)
    table.add_column("Calc Price/Row", style="magenta")
    table.add_column("Calc Accuracy/Row", style="green")
    table.add_column("Calc time/Row", style="green")
    table.add_column("Total Available Annotator", justify="right", style="green")

    for row in market_place_data:
        table.add_row(
            str(row[0]["partner_keyname"]),
            str(row[0]["calc_price_per_row"]),
            str(row[0]["calc_accuracy_per_row"]),
            str(row[0]["calc_time_per_row"]),
            str(row[0]["n_annotators"]),
        )
    console = Console()
    console.print(table)


def call_recommendation_API(annotation_type, project_id, task_type, retry_count=0):
    retry_count = retry_count + 1
    if retry_count > 3:
        raise typer.Exit()
    # take user CLI input
    price_point = int(typer.prompt("Total Point for Price?"))
    time_point = int(typer.prompt("Total Point for Time?"))
    accuracy_point = int(typer.prompt("Total Point for Accuracy?"))

    if price_point + time_point + accuracy_point > 12:
        console.warning("You have exceeded the maximum score")
        call_recommendation_API(
            annotation_type, project_id, task_type, retry_count=retry_count
        )
    elif price_point + time_point + accuracy_point < 12:
        console.warning("Total sum of points is less than 12")
        call_recommendation_API(
            annotation_type, project_id, task_type, retry_count=retry_count
        )

    # recommendation API call
    payload = {
        "annotation_type": annotation_type,
        "n_records": 500,  # hard coded for now
        "project_id": project_id,
        "task_type": task_type,
        "user_accuracy_points": accuracy_point,
        "user_price_points": price_point,
        "user_time_points": time_point,
    }

    token = get_token()
    headers = {"Authorization": str(token), "content-type": "application/json"}
    done = False
    with rich_console.status("[bold green]Working on tasks...") as status:
        while not done:
            post_req_result = requests.post(
                MARKETPLACE_RECOMMENDATION_URL,
                headers=headers,
                data=json.dumps(payload),
            )
            if post_req_result.status_code == 200:
                done = True
    return post_req_result.json()


def select_market_place_recommendation(
    annotation_type, project_id, task_type, choice: str = "1"
):
    if choice == "1":
        recommended_market_place = call_recommendation_API(
            annotation_type, project_id, task_type
        )
        recommended_market_place_list = recommended_market_place["data"]
        show_table(recommended_market_place_list)

        # select multiple market place
        selected_market_place = inquirer.prompt(
            [
                inquirer.Checkbox(
                    "market_place",
                    message="Select market place",
                    choices=[
                        x[0]["partner_keyname"] for x in recommended_market_place_list
                    ],
                )
            ]
        )
    elif choice == "2":
        show_table(market_place_list)
        # select multiple market place
        selected_market_place = inquirer.prompt(
            [
                inquirer.Checkbox(
                    "market_place",
                    message="Select market place",
                    choices=[x[0]["partner_keyname"] for x in market_place_list],
                )
            ]
        )
    return selected_market_place["market_place"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['alectiocli',
 'alectiocli.src',
 'alectiocli.src.experiment',
 'alectiocli.src.hybrid_labeling',
 'alectiocli.src.project']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'alectio-sdk>=0.6.21,<0.7.0',
 'inquirer>=2.9.2,<3.0.0',
 'requests>=2.27.1,<3.0.0',
 'rich>=12.2.0,<13.0.0',
 'ruamel.yaml>=0.17.21,<0.18.0',
 'typer[all]>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['alectio-cli = alectiocli.main:app']}

setup_kwargs = {
    'name': 'alectiocli',
    'version': '0.1.2',
    'description': '',
    'long_description': '',
    'author': 'Adwitiya',
    'author_email': 'adwitiya.trivedi@alectio.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import typer
from pathlib import Path
from .utils import get_config_path
from .config import SSO_URL


def check_token_file():
    config_path = get_config_path()
    token_path = config_path / ".token"
    if not token_path.exists():
        return False
    else:
        return True


def store_token(token):
    config_path = get_config_path()
    token_path = config_path / ".token"
    token_path.write_text(token)
    typer.echo("Successfully stored token.")
    return True


def retrieve_token(refresh):
    token = check_token_file()
    if (not token) or (refresh):
        typer.echo("Logging in to Alectio, retrieving token")
        typer.launch(SSO_URL)
        token = typer.prompt("Token ID: ")
        status = store_token(token)
    else:
        status = True
        typer.echo("Token already exists!")
    return status



import json
import requests
from ..utils import get_token
from ..config import DELETE_PROJECT_URL
from alectiocli.src import console
from rich.console import Console

rich_console = Console()


def delete_project(proj_id: str):

    try:
        with rich_console.status("[bold green]Deleting your project...") as status:
            payload = {"project_id": proj_id}
            token = get_token()
            headers = {"Authorization": str(token)}
            post_req_result = requests.get(
                DELETE_PROJECT_URL,
                headers=headers,
                data=json.dumps(payload),
            )

    except:
        console.error("Something wrong happend !")
        console.info(
            "Make sure, the proj_id is correct.\nuse `alectio-cli project list` to get the correct proj_id"
        )

    if post_req_result.status_code == 200:
        if post_req_result.json()["status"] != "success":
            console.error("Something wrong happend !")
            console.info(
                "Make sure, the proj_id is correct.\nuse `alectio-cli project list` to get the correct proj_id"
            )

        console.success(f"Project with ID: {proj_id} deleted successfully")
    else:
        console.error("Something wrong happend !")
        console.info(
            "Make sure, the proj_id is correct.\nuse `alectio-cli project list` to get the correct proj_id"
        )
